//Copyright © 2013 YU Yezhao & Han JinLin, Reseach Group for Compact Object and Diffuse Medium, NAOC
/*
    This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the license, or (at your option) any later version.

    This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY: without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>
#include <time.h>
#include "cpgplot.h"
#include "path.h"

#define PI 3.141592654

char *strptime(const char*buf, const char *format, struct tm*timeptr);
static void sky();
void palett(int TYPE, float CONTRA, float BRIGHT);
int beam();
void st();

int num_cal;
float min,max,X,Y;
float llo,lla; //observations' longitude & latitude
float LST;//local sidereal time
int settime,ltyear,ltmon,ltday,lth,ltm,bjyear,bjmon,bjday,uth,utm,uts;
int run;
int I,nt,ns,pulsar,np,band,obj;
long n1,n,m,nns;
double delt;
float *galaxy, *background, CONTRA, BRIGHT,*A,*H,d1,d2,ra,dec,*longitude,*latitude;
float SA,SH,*sx,*sy;
float *pra,*prah,*pram,*pras,*pdec,*pdecd,*pdecm,*pdecs,*pf,*pfa,pa,ph,*px,*py;//coordinate parameters of pulsars
float *rrah,*rram,*rras,*rdecd,*rdecm,*rdecs;//coordinate parameters of calibrators
float *rra,*rdec;//calibrators' coordinate parameters, all in degree
float *rf,*rfa;//store the flux of calibrators
double L,B;
float tr[6]={0.1,0.002,0.0,0.1,0.0,0.002};
char *tele,*teles[500],*srs[500],*pn[3000],*srsa[500],*pna[3000],*bgpath,*obpath[1000],dph[100];
int nb,no;
float mbg[1000];
char *bg[1000];
char *back_unit[1000];
float back_freq[1000];
float sfreq_max,sfreq_min,pfreq_max,pfreq_min;
float tela[3]={0.0,0.0,0.0,};
float telh[3]={45.0,0.0,0.0};
float equax[33][400],equay[33][400];
int equa_num[33];

int main(int argc,char *argv[])
{
  int i;
  n=1000000;
  m=500;
  
  for(i=0;i<3000;i++)
    {
      pn[i]=calloc(m+1,sizeof(float));
      pna[i]=calloc(m+1,sizeof(float));
    }
  for(i=0;i<500;i++)
    {
      teles[i]=calloc(m+1,sizeof(float));
      srs[i]=calloc(m+1,sizeof(float));
      srsa[i]=calloc(m+1,sizeof(float));
    }
  for (i=0;i<1000;i++)
    {
      obpath[i]=calloc(100,sizeof(int));
      bg[i]=calloc(10000,sizeof(float));
      back_unit[i]=calloc(100,sizeof(float));
    }
  tele=calloc(m+1,sizeof(float));
  galaxy=calloc(10*n,sizeof(float));
  background=calloc(n+1,sizeof(float));
  A=calloc(n+1,sizeof(float));
  H=calloc(n+1,sizeof(float));
  bgpath=calloc(100,sizeof(int));

  prah=calloc(3001,sizeof(float));
  pram=calloc(3001,sizeof(float));
  pras=calloc(3001,sizeof(float));
  pdecd=calloc(3001,sizeof(float));
  pdecm=calloc(3001,sizeof(float));
  pdecs=calloc(3001,sizeof(float));
  pf=calloc(3001,sizeof(float));
  pfa=calloc(3001,sizeof(float));
  pra=calloc(3001,sizeof(float));
  pdec=calloc(3001,sizeof(float));

  px=calloc(3001,sizeof(float));
  py=calloc(3001,sizeof(float));

  latitude=calloc(m+1,sizeof(float));
  longitude=calloc(m+1,sizeof(float));

  rrah=calloc(m+1,sizeof(int));
  rram=calloc(m+1,sizeof(int));
  rras=calloc(m+1,sizeof(int));
  rdecd=calloc(m+1,sizeof(int));
  rdecm=calloc(m+1,sizeof(int));
  rdecs=calloc(m+1,sizeof(int));
  rf=calloc(m+1,sizeof(float));
  rfa=calloc(m+1,sizeof(float));
  rra=calloc(m+1,sizeof(float));
  rdec=calloc(m+1,sizeof(float));

  sx=calloc(m+1,sizeof(float));
  sy=calloc(m+1,sizeof(float));

  //=================== Read the parameter & set ===================================
  band=0;
  obj=0; 
  run=1;
  delt=0;
  settime=0;
  int j;
  char *line[100],mark[64];
  char *freq_unit;
  float freq;
  num_cal=0;
  freq_unit=calloc(100,sizeof(float));
  for(i=0;i<100;i++)
    {
      line[i]=calloc(100,sizeof(float));
    }
  if(access("radiosky.input",0)!=0)
    {
      printf("You must provide the file \"radiosky.input\"\n");
      printf("See Example: %s/databases/radiosky.input0\n",mph);
      printf("Make such a radiosky.input in current directory\n");
      return 0;
    }
  FILE *fp;
  if((fp=fopen("radiosky.input","r"))==NULL)
    {
      printf("Cannot open radiosky.input.\n");
      printf("See Example: %s/databases/radiosky.input0\n",mph);
      printf("Make such a radiosky.input in current directory\n");
      return 0;
    }
  for(i=0;!feof(fp);i++)
    {
      fgets(line[i],100,fp);
      sscanf(line[i],"%s %*s",mark);
      if(strstr(mark, "vat") != NULL)
	{
	  sscanf(line[i],"%*s %s",tele);
        }
      else if(strstr(mark,"fre"))
	{
	  sscanf(line[i],"%*s %f %s",&freq,freq_unit);
	  if(strcmp(freq_unit,"MHz")==0)
	    {
	      freq=freq;
	    }
	  else if(strcmp(freq_unit,"GHz")==0)
	    {
	      freq=freq*1000;
	    }
	  else
	    {
	      printf("frequency unit ERROR!\n");
	      return 0;
	    }
	}
      else if(strstr(mark,"cal"))
	{
	  num_cal++;
	  obj++;
	  sscanf(line[i],"%*s %s",obpath[obj]);
	  if(access(obpath[obj],0)!=0)
	    {
	      obj--;
	      num_cal--;
	    }
	}
      else if(strstr(mark,"lis"))
	{
	  obj++;
	  sscanf(line[i],"%*s %s",obpath[obj]);
	  if(access(obpath[obj],0)!=0)
	    obj--;
	}
    }
  fclose(fp);

  printf(" Show radio sky at %s ", tele);
  
  if(num_cal==0)
    {
      if(obj>0)
	{
	  strncpy(obpath[2],obpath[1],100);
	  obj++;
	}
      strncpy(obpath[1],mph,50);
      strcat(obpath[1],"/databases/cal.list");
    }

  nb=0;
  char dph[100];

  strncpy(dph,mph,100);
  strcat(dph,"/databases/background.list");
  if((fp=fopen(dph,"r"))==NULL)
    {
      printf("Cannot open background.list\n");
      return 0;
    }
  fscanf(fp,"%*[^\n]%*c");
  for(i=0;!feof(fp);i++)
    {
      fscanf(fp,"%s %f %s %*[^\n]\n",bg[i],&back_freq[i],back_unit[i]);
      if(strcmp(back_unit[i],"MHz")==0)
	{
	  mbg[i]=fabs(back_freq[i]-freq);
	}
      else if(strcmp(back_unit[i],"GHz")==0)
	{
	  mbg[i]=fabs(back_freq[i]*1000-freq);
	}
      else
	{
	  printf("Unit ERROR!\n");
	  return 0;
	}
      nb++;
    }
  fclose(fp);

  for(i=0;i<nb-1;i++)
    {
      if(mbg[band]<=mbg[i+1])
	band=band;
      else
	band=i+1;
    }
  printf("at %.2f %s as the background\n",back_freq[band],back_unit[band]);

  if(argc>1)
    {
      for(i=1;i<argc;i++)
	{
	  if (strcmp(argv[i],"-t")==0)
	    {
	      settime=1;
	      run=0;
	    }
	}
    }

  /*=============== read location information ======================*/
  char *obs_line[500];
  for(i=0;i<m;i++)
    obs_line[i]=calloc(m,sizeof(float));
  nt=0;
  FILE *filein1;
  strncpy(dph,mph,100);
  strcat(dph,"/databases/observatories.info");
  if((filein1=fopen(dph,"r"))==NULL)
    {
      printf("Cannot open observatories.info\n");
      return 0;
    }
  fscanf(filein1,"%*[^\n]%*c");
  fscanf(filein1,"%*[^\n]%*c");
  fscanf(filein1,"%*[^\n]%*c");
  for(i=0;i<m && !feof(filein1);i++)
    {
      fgets(obs_line[i],100,filein1);
      sscanf(obs_line[i],"%s %f %f \n",teles[i],&longitude[i],&latitude[i]);
      nt=nt+1;
    }
  fclose(filein1);

 //=================================================================

  FILE *filein;
  strncpy(dph,mph,100);
  strcat(dph,"/databases/");
  strcat(dph,bg[band]);
  if((filein=fopen(dph,"r"))==NULL)
    {
      printf("Cannot open background data file.\n");
      return 0;
    }
 
  fscanf(filein,"%f %f\n",&min,&max);
  fscanf(filein,"%f %f\n",&X,&Y);
  for(i=0;i<X*Y && !feof(filein);i++)
    {
      fscanf(filein,"%f",&galaxy[i+1]);
    }
  fclose(filein);

   /*=============== read first object information =============*/
  float obj_freq,display_freq,diff_freq;
  if(strcmp(back_unit[band],"MHz")==0)
    {
      display_freq=back_freq[band];
    }
  else
    {
      display_freq=back_freq[band]*1000;
    }
  ns=0;
  if(obj>=1)
    {
      FILE *filein2;

      if((filein2=fopen(obpath[1],"r"))==NULL)
	{
	  printf("Cannot open the first object file.\n");
	  return 0;
	}
      fscanf(filein,"%*[^\n]%*c");
      fscanf(filein,"%*[^\n]%*c");
      for(i=0;i<3000 && !feof(filein2);i++)
	{
	  fscanf(filein2,"%*f %*s %*f:%*f:%*f %*f:%*f:%*f %f %*f\n",&obj_freq);
	  if(i==0)
	    diff_freq = obj_freq-display_freq;
	  if(fabs(diff_freq)>=fabs(obj_freq-display_freq))
	    diff_freq = obj_freq-display_freq;
	}
      fclose(filein2);
      if((filein2=fopen(obpath[1],"r"))==NULL)
	{
	  printf("Cannot open the first object file.\n");
	  return 0;
	}
      fscanf(filein,"%*[^\n]%*c");
      fscanf(filein,"%*[^\n]%*c");
      for(i=0;i<3000 && !feof(filein2);i++)
	{
	  fscanf(filein2,"%*f %s %f:%f:%f %f:%f:%f %f %f\n",srs[i],&rrah[i],&rram[i],&rras[i],&rdecd[i],&rdecm[i],&rdecs[i],&obj_freq,&rf[i]);
	  if(obj_freq==display_freq+diff_freq)
	    {
	      ns=ns+1;
	      rra[i]=(rrah[i]+rram[i]/60.0+rras[i]/3600.0)*15.0;
	      rdec[i]=abs(rdecd[i])+rdecm[i]/60.0+rdecs[i]/3600.0;
	      if(rdecd[i]<0)
		rdec[i]=-rdec[i];
	      if(ns==1)
		{
		  sfreq_max=rf[i];
		  sfreq_min=rf[i];
		}
	      else
		{
		  if(sfreq_max<=rf[i])
		    sfreq_max=rf[i];
		  if(sfreq_min>=rf[i])
		    sfreq_min=rf[i];
		}
	    }
	  else
	    i--;
	}
      fclose(filein2);
    }
  if (obj>1)
    {
      np=0;
      FILE *filein3;

      if((filein3=fopen(obpath[2],"r"))==NULL)
	{
	  printf("Cannot open %s\n",obpath[2]);
	  return 0;
	}
      fscanf(filein,"%*[^\n]%*c");
      fscanf(filein,"%*[^\n]%*c");
      for(i=0;i<3000 && !feof(filein3);i++)
	{
	  fscanf(filein3,"%*f %*s %*f:%*f:%*f %*f:%*f:%*f %f %*f\n",&obj_freq);
	  if(i==0)
	    diff_freq=obj_freq-display_freq;
	  if(fabs(diff_freq)>=fabs(obj_freq-display_freq))
	    diff_freq=obj_freq-display_freq;
	}
      fclose(filein3);

      if((filein3=fopen(obpath[2],"r"))==NULL)
	{
	  printf("Cannot open %s\n",obpath[2]);
	  return 0;
	}
      fscanf(filein,"%*[^\n]%*c");
      fscanf(filein,"%*[^\n]%*c");
      for(i=0;i<3000 && !feof(filein3);i++)
	{
	  fscanf(filein3,"%*f %s %f:%f:%f %f:%f:%f %f %f\n",pn[i],&prah[i],&pram[i],&pras[i],&pdecd[i],&pdecm[i],&pdecs[i],&obj_freq,&pf[i]);
	  if(obj_freq==display_freq+diff_freq)
	    {
	      pra[i]=(prah[i]+pram[i]/60.0+pras[i]/3600.0)*15.0;
	      pdec[i]=abs(pdecd[i])+pdecm[i]/60+pdecs[i]/3600.0;
	      if(pdecd[i]<0)
		pdec[i]=-pdec[i];
	      np=np+1;
	      if(np==1)
		{
		  pfreq_max=pf[i];
		  pfreq_min=pf[i];
		}
	      else
		{
		  if(pfreq_max<=pf[i])
		    pfreq_max=pf[i];
		  if(pfreq_min>=pf[i])
		    pfreq_min=pf[i];
		}
	    }
	  else
	    i--;
	}
      fclose(filein3);
    }
  if(obj>2)
    {
      printf("The radiosky will just display the first two object lists.\n");
    }

  for(i=0;i<nt;i++)
    {
      if(strstr(teles[i],tele)!= NULL)
	{
	  llo=longitude[i];
	  lla=latitude[i];
	  strncpy(tele,teles[i],20);
	  break;
	}
    }
  if(i==nt){
    printf("The observatory are not in the list observatories.info. \n");
    printf("Please check name or add its longitude and latitude.\n");
    return 0;
  }

  /*====================== read pulsar list, if necessary ===================*/
  if (settime==1)
    st();

  //=============== translate equatorial axis to horizon coordinate ========
  float equah,equaa,begin_point;
  for(i=0;i<33;i++)
    equa_num[i]=0;
  if(lla>=0)
    begin_point=PI;
  else
    begin_point=-PI;
  for(i=0;i<9;i++)
    {
      for(j=0;j<400;j++)
	{
	  equah=acos(sin(lla*PI/180)*sin((80-20*i)*PI/180)+cos(lla*PI/180)*cos((80-20*i)*PI/180)*cos(PI*j/200-begin_point));
	  if(equah>=0 && equah<=PI/2)
	    {
	      equaa=atan2(-cos((80-20*i)*PI/180)*sin(PI*j/200-begin_point),(sin((80-20*i)*PI/180)*cos(lla*PI/180)-cos((80-20*i)*PI/180)*sin(lla*PI/180)*cos(PI*j/200-begin_point)));
	      equax[i][equa_num[i]]=(200-sin(equaa)*400*equah/PI)*0.002+0.1;
	      equay[i][equa_num[i]]=(200-cos(equaa)*400*equah/PI)*0.002+0.1;
	      equa_num[i]++;
	    }
	}
    }
  for(i=9;i<33;i++)
    {
      for(j=0;j<400;j++)
	{
	  equah=acos(sin(lla*PI/180)*sin(PI*j/200-begin_point/2)+cos(lla*PI/180)*cos(PI*j/200-begin_point/2)*cos((i-9)*PI/12));
	  if(equah>=0 && equah<=PI/2)
	    {
	      equaa=atan2(-cos(PI*j/200-begin_point/2)*sin((i-9)*PI/12),(sin(PI*j/200-begin_point/2)*cos(lla*PI/180)-cos(PI*j/200-begin_point/2)*sin(lla*PI/180)*cos((i-9)*PI/12)));
	      equax[i][equa_num[i]]=(200-sin(equaa)*400*equah/PI)*0.002+0.1;
	      equay[i][equa_num[i]]=(200-cos(equaa)*400*equah/PI)*0.002+0.1;
	      equa_num[i]++;
	    }
	}
    }

  /*=================== open window and display =====================*/

  int open;
  open = cpgopen("? ");
  if (open == 0)
    return 0;
  cpgpap(7.3,1);
  cpgsvp(0.0, 1.0, 0.0, 1.0);
  cpgswin(0.05,0.95,0.968,0.068);

  beam();
  sky();
  beam();
  while(run==1)
    {
      sleep(5); 
      sky();
      beam();
      
    }
  if(run==0)
    {
      sky();
      beam();
    }
  cpgend();
  cpgclos();
  return 0;
}

//=============================================================
//==========================================================
//=========  subfunction sky()  ========================
//==========================================================
//=============================================================

static void sky()
{ 
  long i;
  char uyear[8],umon[4],uday[4],uh[4],um[4],us[4],sh[4],sm[4],ss[4],bh[4];
  char utc[20]="UTC: ",lst[20]="LST: ",bjt[20]="BJT: ";
  time_t t=time(NULL),start;
  struct tm *timeinfo,tm;
  time(&t);
  timeinfo=gmtime(&t);
  long tjd,td,st;
  double d;
  uth=timeinfo->tm_hour;
  utm=timeinfo->tm_min;
  uts=timeinfo->tm_sec;
  /*=================== calculate the local sidereal time =================*/
  if (settime==0)
    {
      timeinfo=localtime(&t);
      bjyear=timeinfo->tm_year+1900;
      bjmon=timeinfo->tm_mon+1;
      bjday=timeinfo->tm_mday;
      sprintf(uyear,"%04d",bjyear);
      sprintf(umon,"%02d",bjmon);
      sprintf(uday,"%02d",bjday);
    }

  if(settime==1)
    {
      char startime[100];
      sprintf(uyear,"%04d",ltyear);
      sprintf(umon,"%02d",ltmon);
      sprintf(uday,"%02d",ltday);
      sprintf(bh,"%02d",lth);
      if(lth>=8)
	sprintf(uh,"%02d",lth-8);
      else
	sprintf(uh,"%02d",lth+16);
      sprintf(um,"%02d",ltm);
      sprintf(us,"%02d",0);
      strcpy(startime,uyear);
      strcat(startime,"-");
      strcat(startime,umon);
      strcat(startime,"-");
      strcat(startime,uday);
      strcat(startime," ");
      strcat(startime,bh);
      strcat(startime,":");
      strcat(startime,um);
      strcat(startime,":");
      strcat(startime,us);
 
      strptime(startime,"%Y-%m-%d %H:%M:%S",&tm);
 
      start=mktime(&tm);
      delt=difftime(t,start);
      t=t-delt;

    }
  	
  tjd=t/86400+40587;	
  td=t%86400;
  d=(tjd-51545.0)/36525;
  st=(long)(24110.54841+(8640184.812866+(0.093104-0.0000062*d)*d)*d+td*1.00273790935+llo*240);
  st=st%86400;
  if (st<0)
  st=st+86400;
  LST=st*PI/43200;
  
  /*==========================================================================*/
  
  /*=============== Horizon coordinate to Equatorial coordinate ==============*/
  float tt;
  float x,y;
  float D;
  for(i=1;i<=160000;i++)
    {
    x=(i-1)%400+1; //from 1 to 400
    y=(float)floor((i-1)/400)+1;  //from 1 to 400
    D=sqrt((x-200)*(x-200)+(y-200)*(y-200));
    if(D>200)
      {
      A[i]=-1;//use radian
      H[i]=-1;//use radian
      background[i]=sqrt(-1);
      }
    else if(D<=200)
      {
	A[i]=atan2((x-200),(y-200))+PI;
	H[i]=PI*D/400;
	if(H[i]>=PI/2)
	  H[i]=PI/2;
	dec=asin(sin(lla*PI/180)*cos(H[i])+cos(lla*PI/180)*sin(H[i])*cos(A[i]))*180/PI;
	tt=atan2(-sin(H[i])*sin(A[i]),(cos(H[i])*cos(lla*PI/180)-sin(lla*PI/180)*sin(H[i])*cos(A[i])));
	ra=(LST-tt)*180/PI;
/*==========================================================================*/

/*================= translate to galactic coordinate system ================*/

	B=asin(sin(27.12825*PI/180)*sin(dec*PI/180)+cos(27.12825*PI/180)*cos(dec*PI/180)*cos((ra-192.85948)*PI/180))*180/PI;
	L=122.931918-atan2(cos(dec*PI/180)*sin((ra-192.85948)*PI/180),(cos(27.12825*PI/180)*sin(dec*PI/180)-sin(27.12825*PI/180)*cos(dec*PI/180)*cos((ra-192.85948)*PI/180)))*180/PI;

	d1=(float)floor((double)(180-L)/(360/X))+1;
	d2=(float)floor((double)(90-B)/(180/Y))+1;
	I=X*(Y-d2)+d1;

	if(fabs(L-d1*360/X)<=fabs(L-(d1+1)*360/X))
	  I=I;
	else
	  I=I+1;
	if(fabs(B-d2*180/Y)<=fabs(B-(d2+1)*180/Y))
	  I=I;
	else
	  I=I+X;
	background[i]=galaxy[I];
      }
    }
/*==========================================================================*/

/*================ calculate calibrators' X & Y =============== */
  nns=0;
  if(obj>=1)
    {
      for(i=0;i<ns;i++)
	{
	  tt=LST-rra[i]*PI/180;
	  SH=acos(sin(lla*PI/180)*sin(rdec[i]*PI/180)+cos(lla*PI/180)*cos(rdec[i]*PI/180)*cos(tt));
	  if(SH>=0 && SH<=PI/2)
	    {
	      SA=atan2(-cos(rdec[i]*PI/180)*sin(tt),(sin(rdec[i]*PI/180)*cos(lla*PI/180)-cos(rdec[i]*PI/180)*sin(lla*PI/180)*cos(tt)));
	      sx[nns]=(200-sin(SA)*400*SH/PI)*0.002+0.1;
	      sy[nns]=(200-cos(SA)*400*SH/PI)*0.002+0.1;
	      srsa[nns]=srs[i];
	      rfa[nns]=rf[i];
	      nns=nns+1;
	    }
	}
    }

/*======================= calculate pulsars' X & Y =========================*/
  int nnp=0;
  if(obj>1)
    {
      for(i=0;i<np;i++)
	{
	  tt=LST-pra[i]*PI/180;
	  ph=acos(sin(lla*PI/180)*sin(pdec[i]*PI/180)+cos(lla*PI/180)*cos(pdec[i]*PI/180)*cos(tt));
	  if(ph>=0 && ph<=PI/2)
	    {
	      pa=atan2(-cos(pdec[i]*PI/180)*sin(tt),(sin(pdec[i]*PI/180)*cos(lla*PI/180)-cos(pdec[i]*PI/180)*sin(lla*PI/180)*cos(tt)));
	      px[nnp]=(200-sin(pa)*400*ph/PI)*0.002+0.1;
	      py[nnp]=(200-cos(pa)*400*ph/PI)*0.002+0.1;
	      pna[nnp]=pn[i];
	      pfa[nnp]=pf[i];
	      nnp=nnp+1;
	    }
	}
    }

/*============= prepare informations that will write on the window ========*/

  char los[50]="Longitude: ";
  char las[50]="Latitude: ";
  char lo[20];
  char la[20];
  char tea[30]="Azimuth: ";
  char teh[30]="Elevation: ";
  char tead[4],team[4],teas[4],tehd[4],tehm[4],tehs[4];
  char bgfreq_unit[100];
  char ymd[50];

  sprintf(lo,"%6.3f",llo);
  sprintf(la,"%6.3f",lla);
  strcat(los,lo);
  strcat(las,la);
  if(settime!=1)
    {
      sprintf(uh,"%02d",uth);
      sprintf(um,"%02d",utm);
      sprintf(us,"%02d",uts);
      if(uth+8 >= 24)
	sprintf(bh,"%02d",uth-16);
      else
	sprintf(bh,"%02d",uth+8);
    }
  sprintf(sh,"%02ld",st/3600);
  sprintf(sm,"%02ld",st/60%60);
  sprintf(ss,"%02ld",st%60);
  strcat(utc,uh);
  strcat(utc,":");
  strcat(utc,um);
  strcat(utc,":");
  strcat(utc,us);
  strcat(bjt,bh);
  strcat(bjt,":");
  strcat(bjt,um);
  strcat(bjt,":");
  strcat(bjt,us);
  strcat(lst,sh);
  strcat(lst,":");
  strcat(lst,sm);
  strcat(lst,":");
  strcat(lst,ss);

  strncpy(ymd,uyear,50);
  strcat(ymd,"-");
  strcat(ymd,umon);
  strcat(ymd,"-");
  strcat(ymd,uday);

  sprintf(tead,"%3.0f",tela[0]);
  sprintf(team,"%02.0f",tela[1]);
  sprintf(teas,"%02.0f",tela[2]);
  sprintf(tehd,"%3.0f",telh[0]);
  sprintf(tehm,"%02.0f",telh[1]);
  sprintf(tehs,"%02.0f",telh[2]);
  strcat(tea,tead);
  strcat(tea,":");
  strcat(tea,team);
  strcat(tea,":");
  strcat(tea,teas);
  strcat(teh,tehd);
  strcat(teh,":");
  strcat(teh,tehm);
  strcat(teh,":");
  strcat(teh,tehs);

  sprintf(bgfreq_unit,"%d",(int)back_freq[band]);
  strcat(bgfreq_unit," ");
  strcat(bgfreq_unit,back_unit[band]);


/*=========================== write informations ===========================*/

  cpgbbuf();
  cpgeras();
  BRIGHT = 0.5;
  CONTRA  = 1.0;
  palett(2,CONTRA,BRIGHT);
  cpgsfs(2);
  cpgimag(background+1,400,400,1,400,1,400,min,max,tr);

  cpgsci(0);
  cpgslw(200);
  cpgsls(1);
  cpgcirc(0.5, 0.5, 0.49);
  cpgcirc(0.5, 0.5, 0.52);
  cpgslw(70);
  cpgcirc(0.5, 0.5, 0.42);

  cpgslw(1);
  cpgsci(1);
  cpgscf(1);
  cpgsch(0.85);
  cpgmtxt("T",-1.0,0.01,0.0,"Observatory:");
  cpgmtxt("T",-1.0,0.15,0.0,tele);
  cpgmtxt("T", -2.2, 0.034, 0.0, los);
  cpgmtxt("T", -3.4, 0.05, 0.0, las);
  cpgsch(0.90);
  cpgmtxt("T",-1.0, 0.98, 1.0, ymd);
  cpgsch(0.85);
  cpgmtxt("T", -2.2, 0.98, 1.0, bjt);
  cpgmtxt("T", -3.4, 0.98, 1.0, utc);
  cpgmtxt("T", -4.6, 0.98, 1.0, lst);
  cpgmtxt("B", -3.6, 0.0, 0.0, "Background:");
  cpgmtxt("B", -3.6, 0.14, 0.0, bgfreq_unit);
  cpgmtxt("B", -3.4, 0.96, 1.0, "Pointing at");
  cpgpt1(0.8, 0.9, 15);

  cpgmtxt("B", -2.2, 0.98, 1.0, tea);
  cpgmtxt("B", -1.0, 0.98, 1.00, teh);

/*==========================================================================*/

//================= Draw equatorial axis ====================
  char *axis_value;
  int dec_value;
  axis_value=calloc(10,sizeof(float));
  cpgsci(8);//3=green,4=blue,5=light blue,6=pink,7=yellow,8=orange
            //9=light green,11=dark blue,12=purple,13=light purple
            //14=gray,15=light gray,16&more then 16=very dark blue
  cpgslw(2);
  cpgsls(4);
  for(i=0;i<33;i++)
    cpgline(equa_num[i],equax[i],equay[i]);
  for(i=0;i<13;i++)
    {
      dec_value=(int)(st/3600+i-6);
      if(dec_value<0)
	dec_value=dec_value+23;
      sprintf(axis_value,"%d",dec_value);
      strcat(axis_value,":");
      strcat(axis_value,sm);
    }

/*=============================== Draw axis ================================*/

  cpgsls(1);
  cpgsci(15);
  cpgslw(5);
  cpgcirc(0.5,0.5,0.4);

  cpgsls(4);
  cpgslw(1);
  for (i=1;i<10;i=i+2)
    {
      cpgcirc(0.5, 0.5, i*0.4/9); 
    }

  cpgpt1(0.5,0.5,2);
 
  cpgsch(0.6);
  int num;
  char str[4];
  for(i=1;i<4;i++)
    {
      num=i*20;
      sprintf(str,"%d",num);
      cpgptxt(0.11+0.0888*i-0.01,0.5,0.0,0.5,str);
      cpgptxt(0.89-0.0888*i+0.01,0.5,0.0,0.5,str);
    }

  cpgsch(0.85);
	
  float X[2];
  float Y[2];

  for (i=0;i<24;i=i+2)
    {
      X[0] = 0.04*sin(i*PI/12+PI)+0.5;
      Y[0] = 0.04*cos(i*PI/12+PI)+0.5;
      X[1] = 0.4*sin(i*PI/12+PI)+0.5;
      Y[1] = 0.4*cos(i*PI/12+PI)+0.5;
      cpgline(2,X,Y);
    }

  cpgsci(1);
  cpgslw(3);
  for(i=0;i<12;i++)
    {
      num = i*30;
      sprintf(str,"%d",num);
      X[1] = 0.41*sin(i*PI/6+PI)+0.5;
      Y[1] = 0.41*cos(i*PI/6+PI)+0.5;
      if(num==0)
	cpgptxt(X[1],Y[1],0.0,0.5,"North");
      else if(num==90)
	cpgptxt(X[1],Y[1],90.0,0.5,"East");
      else if(num==180)
	cpgptxt(X[1],Y[1]+0.01,0.0,0.5,"South");
      else if(num==270)
	cpgptxt(X[1],Y[1],270.0,0.5,"West");
      else
	cpgptxt(X[1],Y[1],i*30,0.5,str);
    }
  cpgslw(1);
  
/*==========================================================================*/

/*========================= Draw first object ==============================*/
  cpgsci(2); //0=black;1=white;2=red;13=purple,
  cpgsfs(1);
  cpgsch(0.7);
  float radius;
  if(obj>=1)
    {
      for(i=0;i<nns;i++)
	{
	  radius=0.001/(sfreq_max-sfreq_min)*rfa[i]+0.005;
	  cpgcirc(sx[i],sy[i],radius);
	  if(nns<=20)
	    {
	      cpgptxt(sx[i],sy[i]+0.018,0.0,0.5,srsa[i]);
	    }
	}
      cpgsch(0.85);
      cpgscf(1);
      cpgcirc(0.14,0.92,0.005);
      cpgmtxt("B",-2.2,0.12,0.0,": ");
      if(num_cal==0)
	cpgmtxt("B",-2.2,0.14,0.0,"cali sources");
      else
	cpgmtxt("B",-2.2,0.14,0.0,obpath[1]);
    }


  /*==========================================================================*/
  
  /*=================== Point Pulsar on the map if necessary =================*/
  cpgsci(1);
  cpgsls(1);
  cpgsfs(1);
  cpgsch(0.7);
  cpgscf(2);
  if (obj>1)
    {
      for(i=0;i<nnp;i++)
	{
	  radius=1.5/(pfreq_max-pfreq_min)*pfa[i]+0.5;
	  cpgsch(radius);
	  cpgpt1(px[i],py[i],2);
	  
	  if(nnp<=20)
	    {
	      cpgsch(0.7);
	      cpgptxt(px[i],py[i]+0.018,0.0,0.5,pna[i]);
	    }
	}
      cpgsch(0.85);
      cpgscf(1);
      cpgpt1(0.14,0.945,2);
      cpgmtxt("B",-1.0,0.12,0.0,": ");
      cpgmtxt("B",-1.0,0.14,0.0,"Objects to obs");
  }
  cpgscf(2);
  cpgsci(1);
  cpgsch(1);
  cpgebuf();
  return;
}

/*========================== colour palette ==================================*/
void palett(int TYPE, float CONTRA, float BRIGHT) 
{
  //
  float GL[2]={0.0, 1.0};
  float GR[2]={0.0, 1.0};
  float GG[2]={0.0, 1.0};
  float GB[2]={0.0, 1.0};
  //
  float RL[9]={-0.5, 0.0, 0.17, 0.33, 0.50, 0.67, 0.83, 1.0, 1.7};
  float RR[9]={0.0, 0.0,  0.0,  0.0,  0.6,  1.0,  1.0, 1.0, 1.0};
  float RG[9]={0.0, 0.0,  0.0,  1.0,  1.0,  1.0,  0.6, 0.0, 1.0};
  float RB[9]={0.0, 0.3,  0.8,  1.0,  0.3,  0.0,  0.0, 0.0, 1.0};
  //
  float HL[5]={0.0, 0.2, 0.4, 0.6, 1.0};
  float HR[5]={0.0, 0.5, 1.0, 1.0, 1.0};
  float HG[5]={0.0, 0.0, 0.5, 1.0, 1.0};
  float HB[5]={0.0, 0.0, 0.0, 0.3, 1.0};
  //
  float WL[10]={0.0, 0.5, 0.5, 0.7, 0.7, 0.85, 0.85, 0.95, 0.95, 1.0};
  float WR[10]={0.0, 1.0, 0.0, 0.0, 0.3,  0.8,  0.3,  1.0,  1.0, 1.0};
  float WG[10]={0.0, 0.5, 0.4, 1.0, 0.0,  0.0,  0.2,  0.7,  1.0, 1.0};
  float WB[10]={0.0, 0.0, 0.0, 0.0, 0.4,  1.0,  0.0,  0.0, 0.95, 1.0};
  //
  float AL[20]={0.0, 0.1, 0.1, 0.2, 0.2, 0.3, 0.3, 0.4, 0.4, 0.5,0.5, 0.6, 0.6, 
0.7, 0.7, 0.8, 0.8, 0.9, 0.9, 1.0};
  float AR[20]={0.0, 0.0, 0.3, 0.3, 0.5, 0.5, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
 0.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0};
  float AG[20]={0.0, 0.0, 0.3, 0.3, 0.0, 0.0, 0.0, 0.0, 0.8, 0.8, 0.6, 0.6, 1.0,
 1.0, 1.0, 1.0, 0.8, 0.8, 0.0, 0.0};
  float AB[20]={0.0, 0.0, 0.3, 0.3, 0.7, 0.7, 0.7, 0.7, 0.9, 0.9, 0.0, 0.0, 0.0,
 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
  //
  if (TYPE==1) 
    //        -- gray scale
    cpgctab(GL, GR, GG, GB, 2, CONTRA, BRIGHT);
  else if (TYPE==2)
    //        -- rainbow
    cpgctab(RL, RR, RG, RB, 9, CONTRA, BRIGHT);
  else if (TYPE==3)
    //        -- heat
    cpgctab(HL, HR, HG, HB, 5, CONTRA, BRIGHT);
  else if (TYPE==4)
    //        -- weird IRAF
    cpgctab(WL, WR, WG, WB, 10, CONTRA, BRIGHT);
  else if (TYPE==5)
    //        -- AIPS
    cpgctab(AL, AR, AG, AB, 20, CONTRA, BRIGHT);
}




void st()
{
  char ymd[50],hm[50];
  time_t t=time(NULL);
  struct tm *timeinfo;
  time(&t);
  timeinfo=localtime(&t);
  settime=1;
  run=0;
  printf("Input year-month-day[%04d-%02d-%02d]:",timeinfo->tm_year+1900,timeinfo->tm_mon+1,timeinfo->tm_mday);
  fgets(ymd,50,stdin);
  if(ymd[0]=='\n')
    {
      time(&t);
      timeinfo=localtime(&t);
      ltyear=timeinfo->tm_year+1900;
      ltmon=timeinfo->tm_mon+1;
      ltday=timeinfo->tm_mday;
    }
  else
    {
      sscanf(ymd,"%d-%d-%d",&ltyear,&ltmon,&ltday);

    }

  while(ltyear<1970 || ltmon<1 || ltmon>12 || ltday<1 || ((ltmon==4 || ltmon==6 || ltmon==9 || ltmon==11) && ltday>30) || ((ltmon==1 || ltmon==3 || ltmon==5 || ltmon==7 || ltmon==8 || ltmon==10 || ltmon==12) && ltday>31) || (ltmon==2 && ltday>29))
    {
      
      printf("Error: number unavailable!\n");
      printf("Input year-month-day[%04d-%02d-%02d]:",timeinfo->tm_year+1900,timeinfo->tm_mon+1,timeinfo->tm_mday);
      fgets(ymd,50,stdin);
      if(ymd[0]=='\n')
	{
	  time(&t);
	  timeinfo=localtime(&t);
	  ltyear=timeinfo->tm_year+1900;
	  ltmon=timeinfo->tm_mon+1;
	  ltday=timeinfo->tm_mday;
	}
      else
	{
	  sscanf(ymd,"%d-%d-%d",&ltyear,&ltmon,&ltday);

	}
    }
  printf("Input Beijing time (hh:mm)[%d:%d]:",timeinfo->tm_hour,timeinfo->tm_min);
  fgets(hm,50,stdin);
  if(hm[0]=='\n')
    {
      time(&t);
      timeinfo=localtime(&t);
      lth=timeinfo->tm_hour;
      ltm=timeinfo->tm_min;
    }
  else
    {
      sscanf(hm,"%d:%d",&lth,&ltm);
    }

  while(lth>=24 || ltm>=60 || lth<0 || ltm<0)
    {
      printf("Error: number unavailable!\n");
      printf("Input Beijing time (hh:mm)[%d:%d]:",timeinfo->tm_hour,timeinfo->tm_min);
      fgets(hm,50,stdin);
      if(hm[0]=='\n')
	{
	  time(&t);
	  timeinfo=localtime(&t);
	  lth=timeinfo->tm_hour;
	  ltm=timeinfo->tm_min;
	}
      else
	{
	  sscanf(hm,"%d:%d",&lth,&ltm);
	}
    };
}



int beam()
{
  int i;
  float tea,teh,x,y,tran;
  char first,sec;
  FILE *fp;
  if(access("pointing.dat",0)!=0)
    {
      time_t t=time(NULL);
      struct tm *timeinfo;
      time(&t);
      timeinfo=localtime(&t);
      fp=fopen("pointing.dat","w");
      fprintf(fp,"Date\t\ttime\tazimuth\televation\n");
      fprintf(fp,"%d-%d-%d\t%d:%d:%d\t%d:%d:%d\t%d:%d:%d\n",timeinfo->tm_year+1900,timeinfo->tm_mon+1,timeinfo->tm_mday,timeinfo->tm_hour,timeinfo->tm_min,timeinfo->tm_sec,(int)tela[0],(int)tela[1],(int)tela[2],(int)telh[0],(int)telh[1],(int)telh[2]);
      fclose(fp);
    }
  if((fp=fopen("pointing.dat","r"))!=NULL)
    {
      fscanf(fp,"%*s %*s %c%*s %c%*s\n",&first,&sec);
      fscanf(fp,"%*d-%*d-%*d  %*d:%*d:%*d %f:%f:%f %f:%f:%f\n",&tela[0],&tela[1],&tela[2],&telh[0],&telh[1],&telh[2]);
      fclose(fp);
      if(first=='e' && sec=='a')
	{
	  for(i=0;i<3;i++)
	    {
	      tran=tela[i];
	      tela[i]=telh[i];
	      telh[i]=tran;
	    }
	}
      else if(first=='a' && sec=='e')
	{
	}
      else
	{
	  printf("ERROR: Parameter in pointing.dat undefined!\n");
	  printf("Example:\n");
	  printf("Date\t\ttime\tazimuth\televation\n");
	  printf("2013-12-16\t12:23:34\t170:0:0\t83:0:0\n");
	  printf("Make such a pointing.dat in current directory\n");
	  return 0;
	}
    }
  else
    {
      printf("Can not open file for azimuth elevation! \n");
      return 0;
    }
  tea=(tela[0]+tela[1]/60+tela[2]/3600)*PI/180;
  teh=(90-(telh[0]+telh[1]/60+telh[2]/3600))*PI/180;

  x=(200-sin(tea)*400*teh/PI)*0.002+0.1;
  y=(200-cos(tea)*400*teh/PI)*0.002+0.1;

  cpgsci(1);//0=black;1=white;2=red
  cpgsls(1);
  cpgslw(2);
  cpgsch(1.5);
  cpgpt1(x,y,15);
  cpgsch(1);
  return 1;
}
